/**
 * JSCAPE namespace.
 */
var jscape = jscape || {};


/**
 * Extends the embedded String type with new functionality.
 * Verifies if current string ends with specified suffix.
 *
 * @param {String} suffix target suffix
 *
 * @return true if current string ends with specified suffix; false otherwise
 *
 * @author Alex Usun
 */
String.prototype.endsWith = function(suffix) {
    var offset = this.length - suffix.length;

    if (offset < 0) {
        return false;
    }

    for (var i = offset; i < this.length; ++i) {
        if (this.charAt(i) != suffix.charAt(i - offset)) {
            return false;
        }
    }

    return true;
};

/**
 * Extends the embedded String type with new functionality.
 * Trims leading and trailing spaces.
 *
 * @return trimmed string
 *
 * @author Alex Usun
 */
String.prototype.trim = function() {
    return this.replace(/(^\s+)|(\s+$)/g, "");
};


/**
 * Replaces {name} variables with specified substitution.
 *
 * @param substitution substitution parameters
 *
 * @author Alex Usun
 */
String.prototype.supplant = function(substitution) {
    return this.replace(/\{([^{}]*)\}/g,
                        function (value, variable) {
                            var replacement = substitution[variable];
                            return typeof replacement === "string" || typeof replacement === "number" ? replacement : value;
                        }
            );
};


jscape.keyCode2Char = function(keyCode) {
    var numPadKey = keyCode >= 96 && keyCode <= 105;
    return numPadKey ? String.fromCharCode(keyCode - 48) : String.fromCharCode(keyCode);
};


/**
 * Checks if specified code is one of the cursor movement/backspace/delete codes.
 *
 * @param {Number} keyCode the key code to verify
 * @param {Boolean} withCtrl Ctrl key is also pressed
 *
 * @return true if code is one of the control codes; false otherwise
 *
 * @author Alex Usun
 */
jscape.isControlKeyCode = function(keyCode, withCtrl) {
    return keyCode === 8 || keyCode === 9 || keyCode === 17 || keyCode === 35 || keyCode === 36 || keyCode === 37 ||
           keyCode === 38 || keyCode === 39 || keyCode === 40 || keyCode === 45 || keyCode === 46 ||
           (withCtrl && (keyCode === 65 || keyCode === 67 || keyCode === 86 || keyCode === 88));
};

/**
 * Checks if specified code is Tab code.
 *
 * @param {Number} keyCode the key code to verify
 *
 * @return true if code is Tab code; false otherwise
 *
 * @author Alex Usun
 */
jscape.isTabKeyCode = function(keyCode) {
    return keyCode === 9;
};


/**
 * Checks if specified code is Return/Enter code.
 *
 * @param {Number} keyCode the key code to verify
 *
 * @return true if code is Return/Enter code; false otherwise
 *
 * @author Alex Usun
 */
jscape.isReturnKeyCode = function(keyCode) {
    return keyCode === 13;
};

/**
 * Checks if specified code is Esc code.
 *
 * @param {Number} keyCode the key code to verify
 *
 * @return true if code is Esc code; false otherwise
 *
 * @author Alex Usun
 */
jscape.isEscKeyCode = function(keyCode) {
    return keyCode === 27;
};


/**
 * Checks if specified code is digit code.
 *
 * @param {Number} keyCode the key code to verify
 * @param {Boolean} withShift Shift key is also pressed
 *
 * @return true if code is digit code; false otherwise
 *
 * @author Alex Usun
 */
jscape.isDigitKeyCode = function(keyCode, withShift) {
    return (!withShift && (keyCode >= 48 && keyCode <= 57)) || (keyCode >= 96 && keyCode <= 105);
};


/**
 * Checks if specified code is alpha code.
 *
 * @param {Number} keyCode the key code to verify
 *
 * @return true if code is alpha code; false otherwise
 *
 * @author Alex Usun
 */
jscape.isAlphaKeyCode = function(keyCode) {
    return keyCode >= 65 && keyCode <= 90;
};


/**
 * Checks if specified code is alpha or digit code.
 *
 * @param {Number} keyCode the key code to verify
 * @param {Boolean} withShift Shift key is also pressed
 *
 * @return true if code is alpha or digit code; false otherwise
 *
 * @author Alex Usun
 */
jscape.isAlphaOrDigitKeyCode = function(keyCode, withShift) {
    return jscape.isAlphaKeyCode(keyCode) || jscape.isDigitKeyCode(keyCode, withShift);
};


/**
 * Returns Error object message.
 *
 * @param {Error} error an error instance
 *
 * @author Alex Usun
 */
jscape.errorMessageFor = function(error) {
    if (error.toString && typeof error.toString === "function") {
        return error.toString();
    } else if (error.message && error.message.length) {
        return error.message;
    } else if (error.description && error.description.length) {
        return error.description;
    }

    return "Unknown error: " + (typeof error);
};


/**
 * A string resource.
 *
 * @param {Object} strings the strings map
 */
jscape.Resources = function(strings) {
    this.strings = strings ? strings : {};
};


jscape.Resources.prototype = {

    /**
     * Returns the key value with optional substitution.
     *
     * @param {String} key the key
     * @param {Object} data the data to substitute
     */
    getString: function(key, data) {
        if (!this.strings[key]) {
            return key;
        }

        var value = this.strings[key];

        return data ? value.supplant(data) : value;
    },

    contains: function(key) {
        return this.strings[key];
    }
};


/**
 * A timer.
 *
 * @param {Function} task a task
 * @param {Number} millis period millis
 * @param {Boolean} repeatable a repeatable task flag
 *
 * @constructor
 *
 * @author Alex Usun
 */
jscape.Timer = function(task, millis, repeatable) {
    this.task = task;
    this.millis = millis;
    this.repeatable = repeatable;
    this.id = null;
};

jscape.Timer.prototype = {

    /**
     * Starts the timer.
     */
    start: function() {
        if (this.repeatable) {
            this.id = setInterval(this.task, this.millis);
        } else {
            this.id = setTimeout(this.task, this.millis);
        }
    },

    /**
     * Cancels the timer.
     */
    cancel: function() {
        if (!this.id) {
            return;
        }

        if (this.repeatable) {
            clearInterval(this.id);
        } else {
            clearTimeout(this.id);
        }

        this.id = null;
    }
};


/**
 * A time interval.
 *
 * @param {Number} value the time interval in millis
 */
jscape.TimeInterval = function(value) {
    var seconds = Math.round(value / 1000);
    this.hours = this._adjustTimeValue(Math.floor(seconds / 3600));
    this.minutes = this._adjustTimeValue(Math.floor(Math.floor((seconds % 3600)) / 60));
    this.seconds = this._adjustTimeValue(Math.floor(Math.floor((seconds % 3600)) % 60));
};

jscape.TimeInterval.prototype = {

    _adjustTimeValue: function(value) {
        return (value >= 10 ? "" : "0") + value;
    }

};


/**
 * A message dialog.
 *
 * @param {String} dialogDivId a div id
 * @param {String} okButtonText OK button text
 * @param {String} dialogClass a dialog class
 *
 * @constructor
 *
 * @author Alex Usun
 */
jscape.MessageDialog = function(dialogDivId, okButtonText, dialogClass) {
    this.okButtonText = okButtonText;

    this.dialog = this._createDialog(dialogDivId);

    this.listener = null;
    this.okButtonClicked = true;

    this._init(dialogClass);
};

jscape.MessageDialog.prototype = {

    DEFAULT_WIDTH: 400,

    /**
     * Shows dialog.
     *
     * @param listener a dialog listener
     */
    show: function(listener) {
        this.listener = listener;
        this.okButtonClicked = false;
        this.dialog.dialog("open");
    },

    /**
     * Hides dialog.
     */
    hide: function() {
        this.dialog.dialog("close");
    },

    /**
     * Sets dialog title.
     *
     * @param {String} value a dialog title
     */
    setTitle: function(value) {
        this.dialog.attr("title", value);
        this.dialog.dialog("option", "title", value);
    },

    /**
     * Sets dialog message.
     *
     * @param {String} value a dialog message
     */
    setMessage: function(value) {
        this.dialog.html(value);
    },

    /**
     * Sets dialog size.
     *
     * @param {Number} width a dialog width
     * @param {Number} height a dialog height
     */
    setSize: function(width, height) {
        this.dialog.dialog("option", "width", width);
        this.dialog.dialog("option", "height", height);
    },

    _createDialog: function(id) {
        var div = $(document.createElement("div"));
        div.attr("id", id);
        div.hide();

        $(document.body).append(div);

        return div;
    },

    _init: function(dialogClass) {
        var view = this;
        this.dialog.dialog({
            dialogClass: dialogClass,
            width: view.DEFAULT_WIDTH,
            autoOpen: false,
            closeOnEscape: false,
            modal: true,
            resizable: false,
            buttons: view._getButtons(),
            close: function() {
                view._onClose();
            }
        });
    },

    _getButtons: function() {
        var view = this;
        var buttons = {};
        buttons[this.okButtonText] = function() { view._onOkButtonClick(); };

        return buttons;
    },

    _onOkButtonClick: function() {
        this.okButtonClicked = true;
        this.listener.onCloseDialog(this);
    },

    _onClose: function() {
        if (!this.okButtonClicked) {
            this._onOkButtonClick();
        }
    }
};


/**
 * A detailed message dialog.
 *
 * @param {String} dialogDivId a div id
 * @param {String} okButtonText an OK button text
 * @param {String} detailsButtonText a details button text
 * @param {String} dialogTemplate a dialog HTML template
 * @param {String} dialogClass a dialog class
 *
 * @constructor
 *
 * @author Alex Usun
 */
jscape.DetailedMessageDialog = function(dialogDivId, okButtonText, detailsButtonText, dialogTemplate, dialogClass) {
    this.okButtonText = okButtonText;
    this.detailsButtonText = detailsButtonText;
    this.dialogTemplate = dialogTemplate ? dialogTemplate : this.DEFAULT_TEMPLATE;

    this.dialog = this._createDialog(dialogDivId);

    this.messageDiv = this.dialog.find("div").first();
    this.detailsDiv = this.dialog.find("div").last();

    this.listener = null;
    this.okButtonClicked = false;

    this._init(dialogClass);
};

jscape.DetailedMessageDialog.prototype = {

    DEFAULT_WIDTH: 400,

    DEFAULT_TEMPLATE: "<div></div><div style='overflow:auto;height:180px;border:1px solid #c9c9c9;margin-top:10px;padding:10px;'></div>",

    /**
     * Shows dialog.
     *
     * @param listener a dialog listener
     */
    show: function(listener) {
        this.listener = listener;
        this.okButtonClicked = false;
        this.detailsDiv.hide();
        this.dialog.dialog("open");
    },

    /**
     * Hides dialog.
     */
    hide: function() {
        this.dialog.dialog("close");
    },

    /**
     * Sets dialog title.
     *
     * @param {String} value a dialog title
     */
    setTitle: function(value) {
        this.dialog.attr("title", value);
        this.dialog.dialog("option", "title", value);
    },

    /**
     * Sets dialog message.
     *
     * @param {String} value a dialog message
     */
    setMessage: function(value) {
        this.messageDiv.html(value);
    },

    /**
     * Sets dialog message.
     *
     * @param {String} value a dialog details
     */
    setDetails: function(value) {
        this.detailsDiv.html(value);
    },

    _createDialog: function(id) {
        var div = $(document.createElement("div"));
        div.attr("id", id);
        div.html(this.dialogTemplate);
        div.hide();

        $(document.body).append(div);

        return div;
    },

    _init: function(dialogClass) {
        var view = this;

        this.dialog.dialog({
            dialogClass: dialogClass,
            width: view.DEFAULT_WIDTH,
            autoOpen: false,
            closeOnEscape: false,
            modal: true,
            resizable: false,
            buttons: view._getButtons(),
            close: function() {
                view._onClose();
            }
        });
    },

    _getButtons: function() {
        var view = this;
        var buttons = {};
        buttons[this.okButtonText] = function() { view._onOkButtonClick(); };
        buttons[this.detailsButtonText] = function() { view._onDetailsButtonClick(); };

        return buttons;
    },

    _onOkButtonClick: function() {
        this.okButtonClicked = true;
        this.listener.onCloseDialog(this);
    },

    _onDetailsButtonClick: function() {
        this.detailsDiv.toggle();
    },

    _onClose: function() {
        if (!this.okButtonClicked) {
            this._onOkButtonClick();
        }
    }

};


/**
 * A confirmation dialog.
 *
 * @param {String} dialogDivId a div id
 * @param {String} confirmButtonText a confirm button text
 * @param {String} cancelButtonText a cancel button text
 *
 * @constructor
 *
 * @author Alex Usun
 */
jscape.ConfirmationDialog = function(dialogDivId, confirmButtonText, cancelButtonText) {
    this.confirmButtonText = confirmButtonText;
    this.cancelButtonText = cancelButtonText;

    this.dialog = this._createDialog(dialogDivId);

    this.listener = null;
    this.confirmButtonClicked = false;
    this.cancelButtonClicked = false;

    this._init();
};

jscape.ConfirmationDialog.prototype = {

    DEFAULT_WIDTH: 400,

    /**
     * Shows dialog.
     *
     * @param listener a dialog listener
     */
    show: function(listener) {
        this.listener = listener;
        this.confirmButtonClicked = false;
        this.cancelButtonClicked = false;

        $('.ui-dialog-buttonset > css3-container').css("top", "113px");
        $('.ui-dialog-buttonset > css3-container').css("left", "286px");

        this.dialog.dialog("open");
    },

    /**
     * Hides dialog.
     */
    hide: function() {
        this.dialog.dialog("close");
    },

    /**
     * Sets dialog title.
     *
     * @param {String} value a dialog title
     */
    setTitle: function(value) {
        this.dialog.attr("title", value);
        this.dialog.dialog("option", "title", value);
    },

    /**
     * Sets dialog message.
     *
     * @param {String} value a dialog message
     */
    setMessage: function(value) {
        this.dialog.html(value);
    },

    /**
     * Sets dialog size.
     *
     * @param {Number} width a dialog width
     * @param {Number} height a dialog height
     */
    setSize: function(width, height) {
        this.dialog.dialog("option", "width", width);
        this.dialog.dialog("option", "height", height);
    },

    _createDialog: function(id) {
        var div = $(document.createElement("div"));
        div.attr("id", id);
        div.hide();

        $(document.body).append(div);

        return div;
    },

    _init: function() {
        var view = this;
        this.dialog.dialog({
            width: view.DEFAULT_WIDTH,
            autoOpen: false,
            closeOnEscape: false,
            modal: true,
            resizable: false,
            buttons: view._getButtons(),
            close: function() {
                view._onClose();
            }
        });
    },

    _getButtons: function() {
        var view = this;
        var buttons = {};
        buttons[this.cancelButtonText] = function() { view._onCancelButtonClick(); };
        buttons[this.confirmButtonText] = function() { view._onConfirmButtonClick(); };

        return buttons;
    },

    _onConfirmButtonClick: function() {
        this.confirmButtonClicked = true;
        this.listener.onConfirmDialog(this);
    },

    _onCancelButtonClick: function() {
        this.cancelButtonClicked = true;
        this.listener.onCancelDialog(this);
    },

    _onClose: function() {
        if (!this.cancelButtonClicked && !this.confirmButtonClicked) {
            this._onCancelButtonClick();
        }
    }
};


/**
 * A progress dialog.
 *
 * @param {String} dialogDivId a dialog div id
 *
 * @constructor
 *
 * @author Alex Usun
 */
jscape.ProgressDialog = function(dialogDivId) {
    this.dialog = this._createDialog(dialogDivId);

    this.progressBar = this.dialog.find("div").first();

    this._init();
};

jscape.ProgressDialog.prototype = {

    DEFAULT_WIDTH: 500,

    DEFAULT_HEIGHT: 80,

    /**
     * Shows dialog.
     */
    show: function() {
        this.updateProgress(0);
        this.dialog.dialog("open");
    },

    /**
     * Hides dialog.
     */
    hide: function() {
        this.dialog.dialog("close");
    },

    /**
     * Sets dialog title.
     *
     * @param {String} value a dialog title
     */
    setTitle: function(value) {
        this.dialog.attr("title", value);
        this.dialog.dialog("option", "title", value);
    },

    /**
     * Sets dialog class.
     *
     * @param {String} value a dialog class
     */
    setClass: function(value) {
        this.dialog.dialog("option", "dialogClass", value);
    },

    /**
     * Sets dialog size.
     *
     * @param {Number} width a dialog width
     * @param {Number} height a dialog height
     */
    setSize: function(width, height) {
        this.dialog.dialog("option", "width", width);
        this.dialog.dialog("option", "height", height);
    },

    /**
     * Updates progress.
     *
     * @param {Number} percents completeness percents
     */
    updateProgress: function(percents) {
        this.progressBar.progressbar("option", "value", percents);
    },

    _createDialog: function(id) {
        var div = $(document.createElement("div"));
        div.attr("id", id);
        div.html("<div></div>");
        div.hide();

        $(document.body).append(div);

        return div;
    },

    _init: function() {
        var view = this;

        this.progressBar.progressbar({ value: 0 });

        this.dialog.dialog({
            width: view.DEFAULT_WIDTH,
            height: view.DEFAULT_HEIGHT,
            autoOpen: false,
            closeOnEscape: false,
            modal: true,
            resizable: false
        });
    }
};


/**
 * A static message progress dialog.
 *
 * @param {String} dialogDivId a dialog div id
 *
 * @constructor
 *
 * @author Alex Usun
 */
jscape.StaticMessageDialog = function(dialogDivId) {
    this.dialog = this._createDialog(dialogDivId);

    this._init();
};

jscape.StaticMessageDialog.prototype = {

    DEFAULT_WIDTH: 400,

    DEFAULT_HEIGHT: 300,

    /**
     * Shows dialog.
     */
    show: function() {
        this.updateProgress(0);
        this.dialog.dialog("open");
    },

    /**
     * Hides dialog.
     */
    hide: function() {
        this.dialog.dialog("close");
    },

    /**
     * Sets dialog title.
     *
     * @param {String} value a dialog title
     */
    setTitle: function(value) {
        this.dialog.attr("title", value);
        this.dialog.dialog("option", "title", value);
    },

    /**
     * Sets dialog message.
     *
     * @param {String} value a dialog message
     */
    setMessageTemplate: function(value) {
        this.messageTemplate = value;
    },

    /**
     * Sets dialog class.
     *
     * @param {String} value a dialog class
     */
    setClass: function(value) {
        this.dialog.dialog("option", "dialogClass", value);
    },

    /**
     * Sets dialog size.
     *
     * @param {Number} width a dialog width
     * @param {Number} height a dialog height
     */
    setSize: function(width, height) {
        this.dialog.dialog("option", "width", width);
        this.dialog.dialog("option", "height", height);
    },

    /**
     * Updates progress.
     *
     * @param {Number} percents completeness percents
     */
    updateProgress: function(percents) {
        var message = this.messageTemplate.supplant({percents: percents});
        this.dialog.html(message);
    },

    _createDialog: function(id) {
        var div = $(document.createElement("div"));
        div.attr("id", id);
        div.hide();

        $(document.body).append(div);

        return div;
    },

    _init: function() {
        var view = this;
        this.dialog.dialog({
            width: view.DEFAULT_WIDTH,
            height: view.DEFAULT_HEIGHT,
            autoOpen: false,
            closeOnEscape: false,
            modal: true,
            resizable: false
        });
    }
};


/**
 * A time message progress dialog.
 *
 * @param {String} dialogDivId a dialog div id
 *
 * @constructor
 *
 * @author Alex Usun
 */
jscape.TimeMessageDialog = function(dialogDivId) {
    this.dialog = this._createDialog(dialogDivId);

    this.startTime = 0;
    this.percents = "";

    this._init();
};

jscape.TimeMessageDialog.prototype = {

    DEFAULT_WIDTH: 400,

    DEFAULT_HEIGHT: 300,

    UPDATE_MILLIS: 1000,

    /**
     * Shows dialog.
     */
    show: function() {
        this.startTime = new Date().getTime();

        this.updateProgress(0);
        this.timer.start();
        this.dialog.dialog("open");
    },

    /**
     * Hides dialog.
     */
    hide: function() {
        this.dialog.dialog("close");
    },

    /**
     * Sets dialog title.
     *
     * @param {String} value a dialog title
     */
    setTitle: function(value) {
        this.dialog.attr("title", value);
        this.dialog.dialog("option", "title", value);
    },

    /**
     * Sets dialog message.
     *
     * @param {String} value a dialog message
     */
    setMessageTemplate: function(value) {
        this.messageTemplate = value;
    },

    /**
     * Sets dialog class.
     *
     * @param {String} value a dialog class
     */
    setClass: function(value) {
        this.dialog.dialog("option", "dialogClass", value);
    },

    /**
     * Sets dialog size.
     *
     * @param {Number} width a dialog width
     * @param {Number} height a dialog height
     */
    setSize: function(width, height) {
        this.dialog.dialog("option", "width", width);
        this.dialog.dialog("option", "height", height);
    },

    /**
     * Updates progress.
     *
     * @param {Number} percents completeness percents
     */
    updateProgress: function(percents) {
        this.percents = percents;
        this._update();
    },

    _createDialog: function(id) {
        var div = $(document.createElement("div"));
        div.attr("id", id);
        div.hide();

        $(document.body).append(div);

        return div;
    },

    _init: function() {
        var view = this;
        this.dialog.dialog({
            width: view.DEFAULT_WIDTH,
            height: view.DEFAULT_HEIGHT,
            autoOpen: false,
            closeOnEscape: false,
            modal: true,
            resizable: false,
            close: function(event, ui) {
                view._onClose();
            }
        });

        this.timer = new jscape.Timer(function(){
                view._update();
        }, this.UPDATE_MILLIS, true);
    },

    _onClose: function() {
        this.timer.cancel();
    },

    _update: function() {
        var substitution = this._getTakenTime();
        substitution["percents"] = this.percents;
        var message = this.messageTemplate.supplant(substitution);
        this.dialog.html(message);
    },

    _getTakenTime: function() {
        var view = this;
        var takenMillis = Math.round(new Date().getTime() - this.startTime);
        return new jscape.TimeInterval(takenMillis);
    }
};


/**
 * A wait cursor dialog.
 *
 * @param {String} template a status line message template
 *
 * @constructor
 *
 * @author Alex Usun
 */
jscape.WaitCursorDialog = function(template) {
    this.statusTemplate = template ? template : this.DEFAULT_TEMPLATE;
};

jscape.WaitCursorDialog.prototype = {

    DEFAULT_TEMPLATE: "{percents}%",

    /**
     * Shows dialog.
     */
    show: function() {
        document.body.style.cursor = "wait";
    },

    /**
     * Hides dialog.
     */
    hide: function() {
        document.body.style.cursor = "default";
    },

    /**
     * Updates progress.
     *
     * @param {Number} percents completeness percents
     */
    updateProgress: function(percents) {
        window.status = this.statusTemplate.supplant({percents: percents});
    }
};


/**
 * AJAX/REST operation error data.
 *
 * @param {String} httpCode an HTTP error code
 * @param {String} httpText a HTTP error text
 * @param {String} serverText server response text
 * @param {Object} status status
 * @param {Object} error error
 *
 * @constructor
 *
 * @author Alex Usun
 */
jscape.ErrorData = function (httpCode, httpText, serverText, status, error) {
    this.httpCode = httpCode;
    this.httpText = httpText;
    this.serverText = serverText;

    var data = $.parseJSON(serverText);
    // todo: set correct values if not JSON
    this.serverCode = data.code;
    this.serverMessage = data.message;
    this.serverDetails = data.details;
    this.status = status;
    this.error = error;
};


/**
 * A REST operation error dialog.
 *
 * @param {String} dialogDivId a div id
 * @param {String} okButtonText an OK button text
 * @param {String} detailsButtonText a details button text
 * @param {String} title the dialog title
 * @param {jscape.Resources} resources a dialog resources
 *
 * @constructor
 *
 * @author Alex Usun
 */
jscape.RestOperationErrorDialog = function(dialogDivId, okButtonText, detailsButtonText, title, resources) {
    this.dialog = new jscape.DetailedMessageDialog(dialogDivId, okButtonText, detailsButtonText);
    this.resources = resources;

    this.listener = null;

    this.dialog.setTitle(title);
};

jscape.RestOperationErrorDialog.prototype = {

    DEFAULT_WIDTH: 400,

    DETAILS_TEMPLATE: "<p><strong>HTTP response: </strong>{httpCode} ({httpText})</p><p><strong>Server response: </strong> {serverCode} ({serverMessage})</p><p>{serverDetails}</p>",

    /**
     * Shows dialog.
     *
     * @param listener a dialog listener
     */
    show: function(listener) {
        this.listener = listener;
        this.dialog.show(this);
    },

    /**
     * Hides dialog.
     */
    hide: function() {
        this.dialog.hide();
    },

    /**
     * Sets the error.
     */
    setError: function(errorData) {
        var message = this.resources.contains(errorData.serverCode)
                ? this.resources.getString(errorData.serverCode)
                : this.resources.getString("SERVER_ERROR");
        this.dialog.setMessage(message);

        var details = this.DETAILS_TEMPLATE.supplant(errorData);
        this.dialog.setDetails(details);
    },

    onCloseDialog: function() {
        this.listener.onCloseDialog();
    }

};


jscape.NullDialog = function() {
    this.listener = null;
};


jscape.NullDialog.prototype = {

    show: function(listener) {
       if (listener) {
           listener.onCloseDialog();
       }
    },

    hide: function() {},

    setError: function() {
    }
    
};


/**
 * AJAX operation.
 *
 * @param {String} method an HTTP method name
 * @param {String} contentType an HTTP request content type
 * @param {String} dataType expected response content type
 * @param {Number} timeoutMillis a connection timeout in milliseconds
 *
 * @constructor
 *
 * @author Alex Usun
 */
jscape.AjaxOperation = function(method, contentType, dataType, timeoutMillis) {
    this.method = method;
    this.contentType = contentType;
    this.dataType = dataType;
    this.timeoutMillis = timeoutMillis;

    this.listener = null;
};

jscape.AjaxOperation.prototype = {

    /**
     * Starts operation.
     *
     * @param listener operation listener
     * @param {String} url an connection URL
     * @param {String} data data to send
     */
    start: function(listener, url, data) {
        this.listener = listener;
        var operation = this;

        jQuery.ajax({
            type: this.method,
            url: url,
            cache: false,
            timeout: operation.timeoutMillis,
            contentType: operation.contentType,
            data: data,
            processData: false,
            dataType: operation.dataType,

            success: function(data, status, request) {
                operation._onSuccess(request, status, data);
            },

            error: function(request, status, error) {
                operation._onError(request, status, error);
            }
        });
    },

    _onSuccess: function(request, status, data) {
        if (request.status === 0) {
            this.listener.onError(this, new jscape.ErrorData("Connection error.", "", "Connection error.", status, null));
        } else {
            this.listener.onCompleted(this, data, status);
        }
    },

    _onError: function(request, status, error) {
        var errorData = this._getErrorData(request, status, error);
        this.listener.onError(this, errorData);
    },

    _getErrorData: function(request, status, error) {
        try {
            return new jscape.ErrorData(request.status,
                                        request.statusText,
                                        request.responseText,
                                        status,
                                        error);
        } catch(e) {
            return new jscape.ErrorData("Browser error", jscape.errorMessageFor(e), "", status, error);
        }
    }
};


/**
 * REST operation.
 *
 * @param {String} method an HTTP method
 * @param {String} contentType an HTTP request content type
 * @param {String} contentType an expected HTTP response content type
 * @param {Number} timeoutMillis a connection timeout in milliseconds
 * @param {Object} progressDialog a progress dialog
 * @param {Object} errorMessageDialog an error message dialog
 *
 * @constructor
 *
 * @author Alex Usun
 */
jscape.RestOperation = function(method, contentType, dataType, timeoutMillis, progressDialog, errorMessageDialog) {
    this.base = new jscape.AjaxOperation(method, contentType, dataType, timeoutMillis);
    this.progressDialog = progressDialog;
    this.errorMessageDialog = errorMessageDialog;

    this.listener = null;
};

jscape.RestOperation.prototype = {

    /**
     * Starts the operation.
     *
     * @param listener an operation listener
     * @param url target URL
     * @param data data to send
     */
    start: function(listener, url, data) {
        this.listener = listener;

        this.progressDialog.show();
        this.base.start(this, url, data);
    },

    onCompleted: function(operation, data) {
        this.progressDialog.hide();
        this.listener.onCompleted(this, data);
    },

    onError: function(operation, errorData) {
        this.progressDialog.hide();
        this.errorMessageDialog.setError(errorData);
        this.errorMessageDialog.show(this);
    },

    onCloseDialog: function() {
        this.errorMessageDialog.hide();
        this.listener.onBroken(this);
    }

};


/**
 * A float field.
 *
 * @param {Object} field a field
 * @param {Number} maxValue max field value
 * @param {Number} fractionLength allowed fraction length
 * @param {String} alertClass an alert class name
 * @param {Number} alertFadePeriodMillis an alert fade period milliseconds *
 *
 * @constructor
 *
 * @author Alex Usun
 */
jscape.FloatField = function(field, maxValue, fractionLength, alertClass, alertFadePeriodMillis) {
    this.field = field;
    this.maxValue = maxValue || 1000000;
    this.fractionLength = fractionLength || 2;
    this.alertClass = alertClass || "";
    this.alertFadeTimer = this._createTimer(alertFadePeriodMillis || 10000);
    this.integer = NaN;
    this.fraction = NaN;
    this.nextField = null;

    this._init();
};

jscape.FloatField.prototype = {

    getValue: function() {
        return this.field.val();
    },

    setValue: function(value) {
        this.field.val(value);
    },

    intVal: function() {
        this._initFloatValue();
        return this.integer * Math.pow(10, this.fractionLength) + this.fraction;
    },

    getIntValue: function() {
        this._initFloatValue();
        return this.integer * Math.pow(10, this.fractionLength) + this.fraction;
    },

    focus: function() {
        this.field.focus();
    },

    show: function() {
        this.field.show();
    },

    hide: function() {
        this.field.hide();
    },    

    _createTimer: function(alertFadePeriodMillis) {
        var view = this;
        return new jscape.Timer(function(){
            view._removeAlert();
        }, alertFadePeriodMillis, false);
    },

    _init: function() {
        var view = this;

        this.field.keydown(function(e){
            return view._onKeydown(e);
        });

        this.field.blur(function(){
            view._adjustFieldValue();
        });
    },

    _onKeydown: function(event) {
        var code = event.keyCode;

        if (jscape.isControlKeyCode(code, event.ctrlKey)) {
            return true;
        }

        if (jscape.isReturnKeyCode(code) && this.nextField) {
            this._adjustFieldValue();
            this.nextField.focus();
            return false;
        }

        if (this._isPointKeyCode(code, event.shiftKey)) {
            return this.getValue().indexOf(".") === -1;
        }

        return jscape.isDigitKeyCode(code, event.shiftKey);
    },

    _adjustFieldValue: function() {
        this.alertFadeTimer.cancel();

        this._initFloatValue();
        var newValue = this.integer + "." + this._formatFraction();

        if (newValue !== this.getValue()) {
            this.field.val(newValue);
            this._alert();
        } else {
            this._removeAlert();
        }
    },

    _isPointKeyCode: function(keyCode, withShift) {
        return keyCode === 110 || (!withShift && keyCode === 190);
    },

    _initFloatValue: function() {
        this._parseFloatValue();
        this._adjustFloatValue();
    },

    _parseFloatValue: function() {
        var text = this.getValue();

        if (!text.length) {
            this.integer = NaN;
            this.fraction = NaN;
            return;
        }

        var pointIndex = text.indexOf(".");
        var hasPoint = pointIndex > -1;
        var integerPart = !hasPoint ? text : text.substring(0, pointIndex);
        this.integer = integerPart.length ? parseInt(text, 10) : NaN;
        var fractionPart = hasPoint && pointIndex < text.length - 1 ? text.substring(pointIndex + 1) : "";

        if (fractionPart.length > this.fractionLength) {
            fractionPart = fractionPart.substring(0, this.fractionLength);
        }

        this.fraction = fractionPart.length ? parseInt(fractionPart, 10) * Math.pow(10, this.fractionLength - fractionPart.length): NaN;
    },

    _adjustFloatValue: function() {
        if (isNaN(this.integer)) {
            this.integer = 0;
        }

        if (isNaN(this.fraction)) {
            this.fraction = 0;
        }

        if (this.integer > this.maxValue && this.fraction === 0) {
            this.integer = this.maxValue;
        } else if (this.integer >= this.maxValue && this.fraction > 0) {
            this.integer = this.maxValue - 1;
        }
    },

    _formatFraction: function() {
        var fractionText = this.fraction.toString();
        var ret = "";
        for (var i = 0, size = this.fractionLength - fractionText.length; i < size; ++i) {
            ret += "0";
        }

        return ret.length > 0 ? ret + fractionText : fractionText;
    },

    _alert: function() {
        this.field.addClass(this.alertClass);
        this.alertFadeTimer.start();
    },

    _removeAlert: function() {
        this.field.removeClass(this.alertClass);
    }    

};


/**
 * An integer field.
 *
 * @param {Object} field a field
 * @param {Number} maxValue max field value
 * @param {String} alertClass an alert class name
 * @param {Number} alertFadePeriodMillis an alert fade period milliseconds
 *
 * @constructor
 *
 * @author Alex Usun
 */
jscape.IntegerField = function(field, minValue, maxValue, alertClass, alertFadePeriodMillis) {
    this.field = field;
    this.minValue = minValue || 0;
    this.maxValue = maxValue || 1000000;
    this.alertClass = alertClass || "";
    this.alertFadeTimer = this._createTimer(alertFadePeriodMillis || 10000);
    this.intValue = NaN;
    this.nextField = null;

    this._init();
};

jscape.IntegerField.prototype = {

    getValue: function() {
        return this.field.val();
    },

    setValue: function(value) {
        this.field.val(value);
    },

    intVal: function() {
        this._initIntValue();
        return this.intValue;
    },

    getIntValue: function() {
        this._initIntValue();
        return this.intValue;
    },

    focus: function() {
        this.field.focus();
    },

    show: function() {
        this.field.show();
    },

    hide: function() {
        this.field.hide();
    },

    setAlert: function(error) {
        if (error) {
            this.field.addClass("ui-state-error");
        } else {
            this.field.removeClass("ui-state-error");
        }
    },

    _createTimer: function(alertFadePeriodMillis) {
        var view = this;
        return new jscape.Timer(function(){
            view._removeAlert();
        }, alertFadePeriodMillis, false);
    },

    _init: function() {
        var view = this;

        this.field.keydown(function(e){
            return view._onKeydown(e);
        });

        this.field.blur(function(){
            view._adjustFieldValue();
        });
    },

    _onKeydown: function(event) {
        var code = event.keyCode;

        if (jscape.isControlKeyCode(code, event.ctrlKey)) {
            return true;
        }

        if (jscape.isReturnKeyCode(code) && this.nextField) {
            this._adjustFieldValue();
            this.nextField.focus();
            return false;
        }

        return jscape.isDigitKeyCode(code, event.shiftKey)
    },

    _adjustFieldValue: function() {
        this.alertFadeTimer.cancel();

        this._initIntValue();
        var newValue = this.intValue.toString();

        if (newValue !== this.getValue()) {
            this.field.val(newValue);
            this._alert();
        } else {
            this._removeAlert();
        }
    },

    _initIntValue: function() {
        this._parseIntValue();
        this._adjustIntValue();
    },

    _parseIntValue: function() {
        var text = this.getValue();
        this.intValue = text.length ? parseInt(text, 10) : NaN;
    },

    _adjustIntValue: function() {
        if (isNaN(this.intValue) || this.intValue < this.minValue) {
            this.intValue = this.minValue;
        }

        if (this.intValue > this.maxValue) {
            this.intValue = this.maxValue;
        }
    },

    _alert: function() {
        this.field.addClass(this.alertClass);
        this.alertFadeTimer.start();        
    },

    _removeAlert: function() {
        this.field.removeClass(this.alertClass);
    }

};


/**
 * A date field.
 *
 * @param {Object} field a field
 *
 * @constructor
 *
 * @author Alex Usun
 */
jscape.DateField = function(field) {
    this.field = field;
    this.nextField = null;
    this._init();
};

jscape.DateField.prototype = {

    dateVal: function(value) {
        if (value) {
            this.field.datepicker("setDate", value);
        }

        return this.field.datepicker("getDate");
    },

    getDateValue: function() {
        return this.field.datepicker("getDate");
    },

    setDateValue: function(value) {
        this.field.datepicker("setDate", value);
    },

    focus: function() {
        this.field.focus();
    },

    setAlert: function(error) {
        if (error) {
            this.field.addClass("ui-state-error");
        } else {
            this.field.removeClass("ui-state-error");
        }
    },

    show: function() {
        this.field.show();
    },

    hide: function() {
        this.field.hide();
    },

    _init: function() {
        var view = this;
        this.field.datepicker({showAnim: ""});

        this.field.keydown(function(e){
            return view._onKeydown(e);
        });
    },

    _onKeydown: function(event) {
        var code = event.keyCode;

        if (jscape.isReturnKeyCode(code) && this.nextField) {
            this.nextField.focus();
            return false;
        }

        return jscape.isTabKeyCode(code);
    }

};

/**
 * A check box.
 *
 * @param {Object} field a check box element
 *
 * @constructor
 *
 * @author Alex Usun
 */
jscape.CheckBox = function(field) {
    this.field = field;
    this.nextField = null;
    this._init();
};

jscape.CheckBox.prototype = {

    isSelected: function() {
        return this.field.attr("checked") ? true : false;
    },

    setSelected: function(value) {
        if (value) {
            this.field.attr("checked", "checked");
        } else {
            this.field.removeAttr("checked");
        }
    },

    getValue: function() {
        return this.field.val();
    },

    focus: function() {
        this.field.focus();
    },

    show: function() {
        this.field.show();
    },

    hide: function() {
        this.field.hide();
    },

    click: function(listener) {
        if (listener) {
            this.field.click(listener);
        } else {
            this.field.click();
        }
    },

    _init: function() {
        var view = this;

        this.field.keydown(function(e){
            return view._onKeydown(e);
        });
    },

    _onKeydown: function(event) {
        var code = event.keyCode;

        if (jscape.isReturnKeyCode(code) && this.nextField) {
            this.nextField.focus();
            return false;
        }

        return true;
    }
};


/**
 * A combo box.
 *
 * @param {Object} field a combo box element
 *
 * @constructor
 *
 * @author Alex Usun
 */
jscape.ComboBox = function(field) {
    this.field = field;
    this.nextField = null;
    this._init();
};

jscape.ComboBox.prototype = {

    getValue: function() {
        var value = this.field.find(":selected").val();
        return value && value.length > 0 ? value : null;
    },

    setValue: function(value) {
        this.field.val(value);
    },

    focus: function() {
        this.field.focus();
    },

    show: function() {
        this.field.show();
    },

    hide: function() {
        this.field.hide();
    },

    _init: function() {
        var view = this;

        this.field.keydown(function(e){
            return view._onKeydown(e);
        });
    },

    _onKeydown: function(event) {
        var code = event.keyCode;

        if (jscape.isReturnKeyCode(code) && this.nextField) {
            this.nextField.focus();
            return false;
        }

        return true;
    }

};


/**
 * A text field.
 *
 * @param {Object} field a text field element
 *
 * @constructor
 *
 * @author Alex Usun
 */
jscape.TextField = function(field) {
    this.field = field;
    this.nextField = null;
    this._init();
};

jscape.TextField.prototype = {

    getValue: function() {
        return this.field.val();
    },

    setValue: function(value) {
        this.field.val(value);
    },

    focus: function() {
        this.field.focus();
    },

    setAlert: function(error) {
        if (error) {
            this.field.addClass("ui-state-error");
        } else {
            this.field.removeClass("ui-state-error");
        }
    },

    show: function() {
        this.field.show();
    },

    hide: function() {
        this.field.hide();
    },

    _init: function() {
        var view = this;

        this.field.keydown(function(e){
            return view._onKeydown(e);
        });
    },

    _onKeydown: function(event) {
        var code = event.keyCode;

        if (jscape.isReturnKeyCode(code) && this.nextField) {
            this.nextField.focus();
            return false;
        }

        return true;
    }

};



